import json
from typing import List

class ObjEvent:
    taskId = ''
    taskPriority = ''
    taskName = ''
    processId = ''
    domain = ''
    businessKey = ''
    owner = ''
    internalBusinessKey = ''
    def __init__(self):
        self.files = []

def getObjEvent(event) -> List[ObjEvent]: 
    objReturn = []
    myList = list(event['Records'])
    for elem in myList:
        objEvent = ObjEvent()
        objMsg = json.loads(elem["body"])

        objEvent.taskId = objMsg["taskId"]
        objEvent.taskPriority = objMsg["taskPriority"]
        objEvent.taskName = objMsg["taskName"]
        objEvent.processId = objMsg["processId"]
        objEvent.domain = objMsg["domain"]
        objEvent.businessKey = objMsg["businessKey"]
        processVariables = objMsg["processVariables"]
        #objEvent.owner = processVariables["owner"]
        objEvent.internalBusinessKey = processVariables["internalBusinessKey"]
        objReturn.append(objEvent)

    return objReturn


from typing import Union
from datetime import datetime
from sirio.business_object import BusinessObject
import requests
import contextlib
import json

from sirio.exceptions import SirioBoServiceNotRetriableException, SirioBoServiceRetriableException, SirioBoServiceTimeoutException


class SirioService():

    _get_bo_ep: str = '' 
    _complete_task_ep: str = ''
    _error_task_ep: str = ''
    

    def __init__(self, 
                 get_bo_ep: str = None, 
                 complete_task_ep: str = None,
                 error_task_ep: str = None, 
                 host_sirio_ep: str = None) -> None:

        """
        SirioCommService Class

        This class is responsible for all the communication to and from Sirio 

        Args:
            get_bo_ep (str): the URL of the endpoint to get a BO, given a businesskey. Alternatively, use only host_sirio_ep
            complete_task_ep (str): the URL of the endpoint to complete a task, given a businesskey and a valid BO. Alternatively, use only host_sirio_ep
            error_task_ep (str): the URL of the endpoint to error a task, given a businesskey and a valid BO. Alternatively, use only host_sirio_ep
            host_sirio_ep (str): the URL of the host for all endpoints. Alternatively, use: get_bo_ep, complete_task_ep and error_task_ep
        """
        if host_sirio_ep is not None:
            self._get_bo_ep = '{host}/sirio/enginebackend/businessobjects/{businessKey}?domain={domain}&taskId={taskId}'.replace('{host}',host_sirio_ep)
            self._complete_task_ep = '{host}/sirio/enginebackend/processes/domains/{domain}/tasks/{taskId}'.replace('{host}',host_sirio_ep)
            self._error_task_ep = '{host}/sirio/enginebackend/processes/domains/{domain}/tasks/{taskId}'.replace('{host}',host_sirio_ep) #Da sistemare quando il servizio sarà disponibile
        else:
            self._get_bo_ep = get_bo_ep
            self._complete_task_ep = complete_task_ep
            self._error_task_ep = error_task_ep


    def retrieveBo(self, businesskey: str, domain: str, taskId: str):
        
        try:
            response = requests.get(
                self._get_bo_ep.replace('{businessKey}', businesskey).replace('{domain}',domain).replace('{taskId}', taskId))
            
            # Check if something went wrong
            response.raise_for_status()
            
            # Get and return the Business Object
            bo = BusinessObject(businessKey=businesskey)
            bo.businessObject = response.json()
            bo.description = bo.businessObject['description']
            bo.subject = bo.businessObject['subject']

            return bo

        except requests.exceptions.Timeout as ex:
            # This is retryable, but we *DON'T* want a retry policy executed in the lambda, to avoid 
            # nasty interaction between SQS visibility_windows and the lambda_timeout setting. 
            # If something goes wrong, the same message could be processed more than once, and 
            # who knows what could happen...
            raise SirioBoServiceTimeoutException(f"Timeout retrieving BO with BusinessKey {businesskey}")
        except requests.HTTPError as ex:
            # In these case we verify that the error is 404. In this case we want not to retry the 
            status_code = ex.response.status_code

            if status_code in [404, 501, 503]:
                raise SirioBoServiceNotRetriableException(f"BO with BusinessKey {businesskey} does not exists. Status {status_code}")

            raise SirioBoServiceRetriableException(f"Error retriving BO with BusinessKey {businesskey}. Status {status_code}")
        except Exception as ex:
            raise SirioBoServiceRetriableException(f"Generic error retriving BO with BusinessKey {businesskey}. Exc -> {ex}")



    def completeTask(self, bo: BusinessObject, domain: str, taskId: str, priority=None, owner=None):
        with contextlib.ExitStack() as stack:
            if len(bo.objects) > 0:
                if 'objects' not in bo.jsonComplete:
                    bo.jsonComplete['objects'] = []
                
                for index, obj in enumerate(bo.objects):
                    if obj.multipartMode:
                        jsonObj = {
                            "id" : obj.id,
                            "extension" : obj.extension,
                            "name" : obj.name,
                            "key" : obj.key,
                            "base64" : obj.base64,
                            "partIndex":index,
                            "params" : obj.params
                        }
                        bo.objects[index].file = stack.enter_context(open(obj.fileName, 'rb'))
                    else:
                            jsonObj = {
                            "id" : obj.id,
                            "extension" : obj.extension,
                            "name" : obj.name,
                            "key" : obj.key,
                            "base64" : obj.base64,
                            "params" : obj.parmas
                        }
                            #'dto': (json, 'application/json')
                    bo.jsonComplete['objects'].append(jsonObj)
            #json_file = stack.enter_context(tempfile.NamedTemporaryFile(delete=False, mode='w+', encoding='utf-8'))
            #json.dump(json_string, json_file, ensure_ascii=False, indent=4)
            #json_file.flush() 
            dto = bo.jsonComplete
            files = []
            payload = ('dto',(None, json.dumps(dto), 'application/json'))
            files.append(payload)
            for index, obj in enumerate(bo.objects):
                if obj.multipartMode:
                    files.append(('files',('{}.{}'.format(obj.name,obj.extension), obj.file, obj.contentType)))
            try:
                responseComplete = requests.post(self._complete_task_ep.replace('{domain}', domain).replace('{taskId}', taskId), files=files)

                # Check if something went wrong. We intercept all 
                # the issues in the response            
                responseComplete.raise_for_status()

                return True


            except requests.exceptions.Timeout as ex:
                # This is retryable, but we *DON'T* want a retry policy executed in the lambda, to avoid 
                # nasty interaction between SQS visibility_windows and the lambda_timeout setting. 
                # If something goes wrong, the same message could be processed more than once, and 
                # who knows what could happen...
                raise SirioBoServiceTimeoutException(f"Timeout persisting BO with BusinessKey {bo.businessKey}")
            except requests.HTTPError as ex:
                # In these case we verify that the error is 404. In this case we want not to retry the 
                status_code = ex.response.status_code

                if status_code in [404, 501, 503]:
                    raise SirioBoServiceNotRetriableException(f"Error persisting BO with BusinessKey {bo.businessKey}. Status {status_code}")

                raise SirioBoServiceRetriableException(f"Error persisting BO with BusinessKey {bo.businessKey}. Status {status_code}")
            except Exception as ex:
                raise SirioBoServiceRetriableException(f"Generic error persisting BO with BusinessKey {bo.businessKey}. Exc -> {ex}")
            
       


class TimeoutException(Exception):
    pass

class RetriableException(Exception):
    pass

class NotRetriableException(Exception):
    pass

class SirioBoServiceTimeoutException(TimeoutException):
    pass

class SirioBoServiceRetriableException(RetriableException):
    pass

class SirioBoServiceNotRetriableException(NotRetriableException):
    pass



import requests
from enum import Enum
import tempfile
import base64
import json
from sirio.utility import *
import contextlib
import os


class TypeValue(Enum):
    string = 1
    date = 2
    numeric = 3

class Object:
    id = ""
    extension = ""
    name = ""
    key = ""
    base64 = ""
    partIndex = 0
    params = []
    file = None
    fileName = None
    multipartMode = True
    contentType = ''
    def __init__(self, key, name, id='', extension='', params: list = [], partIndex=-1, multipartMode=True ):
        self.id = id
        self.name = name
        self.extension = extension
        self.key = key
        self.params = params
        self.partIndex = partIndex
        self.multipartMode = multipartMode

    def createByBase64(self, base64_string):
        """
        Crea file a partire da un bytearray codificato in Base64.

        :param base64_string: Stringa contenente i dati codificati in Base64.
        """
        file_data = base64.b64decode(base64_string)
        fileName = f"{self.name}.{self.extension}"
        temp_file_path = os.path.join(tempfile.gettempdir(), fileName)
        with open(temp_file_path, 'wb') as temp_file:
            #Converte la stringa base64 in byteArray
            temp_file.write(file_data)

        self.fileName = temp_file.name
        self.contentType = getContentTypeFromName(fileName)
        


    def createByStream(self, file_stream):
        """
        Crea file a partire da uno stream.

        :param file_stream: Stream del file.
        """
        # Crea un file temporaneo con il nome e l'estensione specificati
        fileName = f"{self.name}.{self.extension}"
        temp_file_path = os.path.join(tempfile.gettempdir(), fileName)

        # Scrivi il contenuto dello stream nel file temporaneo
        with open(temp_file_path, 'wb') as temp_file:
            file_stream.seek(0)  # Assicura che lo stream sia alla posizione iniziale
            temp_file.write(file_stream.read())

        self.fileName = temp_file.name
        self.contentType = getContentTypeFromName(fileName)

    def createByJson(self, str_son):
        """
        Crea file a partire da un json.

        :param str_son: json del file.
        """
        ENCODING = 'utf-8'
        self.extension = 'json'
        with tempfile.NamedTemporaryFile(delete=False, mode='w+', encoding='utf-8') as json_file:
            json.dump(str_son, json_file, ensure_ascii=False, indent=4)
            json_file.flush()  
        self.fileName = json_file.name
        self.contentType = 'application/json'
        """
        if not self.multipartMode:
            with open(json_file.name, 'rb') as json_file:
                encoded = base64.b64encode(json_file.read())
            self.base64 = encoded.decode(ENCODING)
        else:
            with open(json_file.name, 'rb') as json_file:
                self.file = json_file
        """
class BusinessObject:
    jsonComplete = {}
    businessObject = {}
    businessKey =  ""
    subject = ""
    description = ""
    urlGetBO = ""
    urlComplete = ""
    objects = []
    def __init__(self, businessKey: str, urlGetBO: str = None, urlComplete: str = None):
        self.jsonComplete = {"businessKey": businessKey}
        self.objects = []
        self.data = {}
        self.urlGetBO = urlGetBO
        self.urlComplete = urlComplete
        if self.urlGetBO is not None:
            try:
                response = requests.get(urlGetBO.replace('{businessKey}', businessKey))
                if response.status_code == 200:
                    self.businessObject = response.json()
                    self.description = self.businessObject['description']
                    self.subject = self.businessObject['subject']
                else:
                    self = None
            except Exception as ex:
                self.businessObject = self.jsonComplete
                raise Exception('Eccezione nel recupero del businessObject: {}'.format(ex))
    
    def getListIdInBind(self, bind:str):
        keys_list = []
        if existsJsonPath(self.businessObject,['data',bind]):# 'data' in self.businessObject['data'] and bind in self.businessObject['data'] and id in self.businessObject['data'][bind] and 'value' in self.businessObject['data'][bind][id] and 'value' in self.businessObject['data'][bind][id]['value']:
                keys_list = list(self.businessObject['data'][bind].keys())
        return keys_list    

    def getValue(self, bind:str, id:str):
        valReturn = None
        try:
            if existsJsonPath(self.businessObject,['data',bind, id, 'value', 'value']):# 'data' in self.businessObject['data'] and bind in self.businessObject['data'] and id in self.businessObject['data'][bind] and 'value' in self.businessObject['data'][bind][id] and 'value' in self.businessObject['data'][bind][id]['value']:
                valReturn = self.businessObject['data'][bind][id]['value']['value']
        except Exception as ex:
            print("Eccezione su getValue [{}][{}] - {}".format(bind, id, ex))
        return valReturn
    
    def setValue(self, bind: str, id: str, value, typeValue=TypeValue.string, descriprion=''):
        if 'data' not in self.jsonComplete:
            self.jsonComplete['data'] = {}
        if bind not in self.jsonComplete['data']:
            self.jsonComplete['data'][bind] = {}
        if id not in self.jsonComplete['data'][bind]:
            self.jsonComplete['data'][bind][id] =  {"dataType": typeValue.name,  "description": descriprion,   "value": {"value": value }, "extendedValue": [] }
        elif self.jsonComplete['data'][bind][id]['value']['value'] != value:
            self.jsonComplete['data'][bind][id]['value']['value'] = value

        if 'data' not in self.businessObject:
            self.businessObject['data'] = {}
        if bind not in self.businessObject['data']:
            self.businessObject['data'][bind] = {}
        if id not in self.businessObject['data'][bind]:
            self.businessObject['data'][bind][id] = {"dataType": typeValue.name,  "description": descriprion,   "value": {"value": value }, "extendedValue": [] }
        elif self.businessObject['data'][bind][id]['value']['value'] != value:
            self.businessObject['data'][bind][id]['value']['value'] = value
    
    def getOutputTask(self, id:str):
        valReturn = ''
        try:
            if existsJsonPath(self.businessObject,['outputTask',id]):
                valReturn = self.businessObject['outputTask'][id]
        except Exception as ex:
            print("Eccezione su getOutputTask [{}] - {}".format(id, ex))
        return valReturn
    
    def deleteOutputTask(self, id:str):
        try:
            if existsJsonPath(self.businessObject,['outputTask',id]):
                self.businessObject['outputTask'].pop(id)
            if existsJsonPath(self.jsonComplete,['outputTask',id]):
                self.jsonComplete['outputTask'].pop(id)
        except Exception as ex:
            print("Eccezione su deleteOutputTask [{}] - {}".format(id, ex))
        
    
    def setOutputTask(self, id: str, value):
        if 'outputTask' not in self.jsonComplete:
            self.jsonComplete['outputTask'] = {}
        if id not in self.jsonComplete['outputTask']:
            self.jsonComplete['outputTask'][id] =  value
        elif self.jsonComplete['outputTask'][id] != value:
            self.jsonComplete['outputTask'][id] = value

        if 'outputTask' not in self.businessObject:
            self.businessObject['outputTask'] = {}
        if id not in self.businessObject['outputTask']:
            self.businessObject['outputTask'][id] = value
        elif self.businessObject['outputTask'][id] != value:
            self.businessObject['outputTask'][id] = value


    def getObject(self, key: str):
        objectsReturn = []
        #object = None
        try:
            if existsJsonPath(self.businessObject,['objects']):
                objList = self.businessObject['objects']
                if objList is not None:
                    objects = list(objList)
                    if objects is not None:
                        for obj in objects:
                            if obj['key'] == key:
                                objectsReturn.append(obj)
        except Exception as ex:
            print("Eccezione su getObject - {}".format(ex))
        return objectsReturn
    
    def uploadObject(self, object: Object):
        self.objects.append(object)
        

    def complete(self, domain, taskId, priority=None, owner=None):
        if self.urlComplete is not None:
            dto = self.jsonComplete #_to_json(self.jsonComplete)
            #Preparo gli object
            objSend = []
            with contextlib.ExitStack() as stack:
                if len(self.objects) > 0:
                    if 'objects' not in self.jsonComplete:
                        self.jsonComplete['objects'] = []
                    
                    for index, obj in enumerate(self.objects):
                        if obj.multipartMode:
                            jsonObj = {
                                "id" : obj.id,
                                "extension" : obj.extension,
                                "name" : obj.name,
                                "key" : obj.key,
                                "base64" : obj.base64,
                                "partIndex":index,
                                "params" : obj.params
                            }
                            self.objects[index].file = stack.enter_context(open(obj.fileName, 'rb'))
                        else:
                                jsonObj = {
                                "id" : obj.id,
                                "extension" : obj.extension,
                                "name" : obj.name,
                                "key" : obj.key,
                                "base64" : obj.base64,
                                "params" : obj.parmas
                            }
                                #'dto': (json, 'application/json')
                        self.jsonComplete['objects'].append(jsonObj)
                json_file = stack.enter_context(tempfile.NamedTemporaryFile(delete=False, mode='w+', encoding='utf-8'))
                #json.dump(json_string, json_file, ensure_ascii=False, indent=4)
                #json_file.flush() 
                files = []
                payload = ('dto',(None, json.dumps(dto), 'application/json'))
                files.append(payload)
                for index, obj in enumerate(self.objects):
                    if obj.multipartMode:
                        files.append(('files',('{}.{}'.format(obj.name,obj.extension), obj.file, obj.contentType)))
                responseComplete = requests.post(self.urlComplete.replace('{domain}', domain).replace('{taskId}', taskId), files=files)
            return responseComplete

def _to_json(obj):
    return json.dumps(obj, indent=4, default=lambda obj: obj.__dict__)





import mimetypes

def existsJsonPath(document, paths: list):
    esiste = True
    subDocument = document
    for sub in paths:
        if sub in subDocument:
            subDocument = subDocument[sub]
        else:
            esiste = False
            break
    return esiste

def getContentTypeFromName(file_name):
    content_type, _ = mimetypes.guess_type(file_name)
    return content_type or "application/octet-stream"  # valore predefinito


from setuptools import setup, find_packages

from distutils.core import setup
setup(
    name='sirio',
    packages=find_packages(),
    version='0.1.23',
    description='Sirio',
    author='Pasquale Rombolà',
    license='BSD',
    author_email='pasquale.rombola@cerved.com',
    url='https://github.com/pask-xx/sirio.git',
    keywords=['sirio', 'template', 'package', ],
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development',
    ],
    install_requires=['requests'],
)


#!/usr/bin/env python

"""Tests for `sirio` package."""


import unittest
import io

from sirio import event

from sirio.business_object import BusinessObject, Object




obj = Object('TEST-PASK','pask', extension='json')
stream = io.BytesIO(b"Questo il contenuto del file.")
obj.createByStream(stream)

url_sirio_get_business_object = 'https://sirio-engine-backend.srv-test.eu-central-1.aws.cervedgroup.com/sirio/enginebackend/businessobjects/{businessKey}'
url_sirio_complete = 'https://sirio-engine-backend.srv-test.eu-central-1.aws.cervedgroup.com/sirio/enginebackend/processes/domains/{domain}/tasks/{taskId}'
bo = BusinessObject('67063d15abcaec5dc6e87bb9',url_sirio_get_business_object, url_sirio_complete)
testJson = {'a':'paskA','b':'paskB'}
obj = Object('TEST-PASK','pask', extension='json')
stream = io.BytesIO(b"Questo il contenuto del file.")
obj.createByStream(stream)
bo.uploadObject(obj)
bo.complete('ir4','6316d202-ba63-4785-84ac-a0415e44177e')

class TestSirio(unittest.TestCase):
    """Tests for `sirio` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""
